package pooldemo;

import java.util.*;
import java.time.*;
import org.apache.commons.pool2.*;
import org.apache.commons.pool2.impl.*;

@SuppressWarnings("rawtypes")
public class Pool2App {
	public static class MyPooledObjectFactory<T> extends BasePooledObjectFactory<ArrayList<T>> {
		public ArrayList<T> create() throws Exception {
			return new ArrayList<T>();
		}
		public PooledObject<ArrayList<T>> wrap(ArrayList<T> obj) {
			return new DefaultPooledObject<ArrayList<T>>(obj);
		}
		public void passivateObject(PooledObject<ArrayList<T>> p) throws Exception {
			p.getObject().clear();
		}
	}
	
	public static void printPoolInfo(GenericObjectPool objectPool) {
		String msg = String.format("active=%d, idle=%d, waiters=%d, created=%d, borrowed=%d, returned=%d",
				objectPool.getNumActive(),
				objectPool.getNumIdle(),
				objectPool.getNumWaiters(),
				objectPool.getCreatedCount(),
				objectPool.getBorrowedCount(),
				objectPool.getReturnedCount());
		System.out.println(msg);
	}
	
	public static GenericObjectPool<ArrayList<String>> objectPool;
	public static void testGenericObjectPool() throws Exception {
		MyPooledObjectFactory<String> objFactory = new MyPooledObjectFactory<String>();
		GenericObjectPoolConfig<ArrayList<String>> config = new GenericObjectPoolConfig<ArrayList<String>>();
		config.setTimeBetweenEvictionRuns(Duration.ofSeconds(10));
		config.setMinEvictableIdleDuration(Duration.ofSeconds(30));
		GenericObjectPool<ArrayList<String>> objectPool =
				new GenericObjectPool<ArrayList<String>>(objFactory, config);
		objectPool.addObjects(4);
		
		Pool2App.objectPool = objectPool;
		ArrayList<String> list = objectPool.borrowObject();
		printPoolInfo(objectPool);
		objectPool.returnObject(list);
		printPoolInfo(objectPool);
	}
	
	public static class MyKeyedPooledObjectFactory<T> extends BaseKeyedPooledObjectFactory<String,ArrayList<T>> {
		public ArrayList<T> create(String key) throws Exception {
			return new ArrayList<T>();
		}
		public PooledObject<ArrayList<T>> wrap(ArrayList<T> value) {
			return new DefaultPooledObject<ArrayList<T>>(value);
		}
		public void passivateObject(String key, PooledObject<ArrayList<T>> p) throws Exception {
			p.getObject().clear();
		}
	}
	
	public static void printPoolInfo(GenericKeyedObjectPool objectPool) {
		String msg = String.format("active=%d, idle=%d, waiters=%d, created=%d, borrowed=%d, returned=%d",
				objectPool.getNumActive(),
				objectPool.getNumIdle(),
				objectPool.getNumWaiters(),
				objectPool.getCreatedCount(),
				objectPool.getBorrowedCount(),
				objectPool.getReturnedCount());
		System.out.println(msg);
	}
	
	public static GenericKeyedObjectPool<String,ArrayList<String>> keyedObjectPool;
	public static void testGenericKeyedObjectPool() throws Exception {
		MyKeyedPooledObjectFactory<String> objFactory = new MyKeyedPooledObjectFactory<String>();
		GenericKeyedObjectPoolConfig<ArrayList<String>> config = new GenericKeyedObjectPoolConfig<ArrayList<String>>();
		GenericKeyedObjectPool<String,ArrayList<String>> keyedObjectPool =
				new GenericKeyedObjectPool<String,ArrayList<String>>(objFactory, config);
		
		Pool2App.keyedObjectPool = keyedObjectPool;
		ArrayList<String> list = keyedObjectPool.borrowObject("name");
		printPoolInfo(keyedObjectPool);
		keyedObjectPool.returnObject("name", list);
		printPoolInfo(keyedObjectPool);
	}
	
	public static void main(String[] args) throws Exception {
		testGenericObjectPool();
		testGenericKeyedObjectPool();
	}
}

